/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
//
// REVISION HISTORY:
// Version 1.6 13/09/18;
//						- Added a Settings rollout containing: a Quiet mode toggle applicable for all browser methods, a Longer UI toggle,
//						  and a Persistent Filters toggle which is applied across all browser methods.
//						- Added a clear filter string button 'x' for all filter edit boxes.
// Version 1.5 05/09/18;
//						- History method now displays folder paths in the dropdown sourced from a stored custom list of the most recently
//						  accessed unique paths opened by the browser. The most recent path is always at the top. The list defaults to the
//						  history items stored in the 3dsmax.ini (if any).
// Version 1.4.1 03/09/18;
//						- Fixed a bug where the script crashed if attempting to browse a folder or subfolders with system access denied.
//						- Override browsing (i.e. using the Browse button) can now be done in the Scene method without a scene already open.
// Version 1.4 31/08/18;
//						- Added an Up button and a Browse button for the Scene method to enable temporary path browsing away from
//						  the current scene file path. Also added a Refresh button to quickly sync the list with the current scene file path.
//						- Missing maxfiles while the UI is open and displaying them in the maxfiles list will now be caught.
//						- Minor UI button changes to Folders method.
//						- Bug fixes.
// Version 1.3 28/08/18;
//						- Added a History browser method for accessing maxfiles based on the XML stored list of recent files, as well
//						  as a dropdown list of the most recently accessed folder paths.
//						- Added an edit field for simple substring name filtering of maxfile items in the list boxes for each browser method.
//						- Added the ability to add the currently open scene to the text file when using the Files browser method.
//						- Added current scene display to the bottom of the UI for all browser methods, with an Explorer browse button.
//						- Added a toolbar icon.
//						- Quiet modes now display errors when attempting to open or merge from newer 3dsMax version files.
//						- Subfolders are now more obviously displayed in the list boxes using "..\" prefix.
//						- Changed Text browser method to now be called "Files" externally.
//						- Removed the "***  Dump To Text File  ***" button from Files browser method (redundant).
//						- Reduced overall height of the UI (since it has filtering now).
// Version 1.2 31/10/14;
//						- You can now Control+click to open a file or Shift+click to merge from a file selected from the listbox for all
//							method types.
// Version 1.1 27/10/14;
//						- Email change for help contact.
// Version 1.0 23/09/14;
//						- First public release.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/

global g_LBTools_MFB

struct s_LBTools_maxfilesBrowser
(

/*==================================================================================
// Structure variables
====================================================================================*/

maxfilesBrowserFloater,
aboutScript_Rollout,
settings_Rollout,
maxfilesBrowserUI_Rollout,
f_openMaxfilesBrowser,
MFB_Pos, --// Floater position
UIheight, --// Adjustable height for the UI based on a settings checkbox
textListHeight,
foldersPathsHeight,
foldersListHeight,
sceneListHeight,
historyListHeight,
persistFilters,
filters_str = "", --// A persistent filters string, if persistent filters option is checked. Only stored in memory - not an INI file
quietMode,
imageFile1 = (getDir #usericons) + "\LBTools_Folder_Image.bmp",
imageFile2 = (getDir #usericons) + "\LBTools_UpArrow_Image.bmp",
historySubfolders = true,
historyPathsIndex = 1, --// Defaults to the recent history files list (not history paths)
maxHistoryPaths = 30, --// Sets a hard limit for how many history paths to store and display in History method
	
/*==================================================================================*/

fn f_maxfilesBrowser =
(
	rollout maxfilesBrowserUI_Rollout "Method Types"
	(
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/	
		-- //	SHARED FUNCTIONS
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		
		fn f_removeLastSlash path_str =
		(
			replace path_str path_str.count 1 ""
		)
		
		fn f_browseScenePath =
		(
			local str_browsePath = maxFilePath
			if str_browsePath != "" do
			(
				str_browsePath = f_removeLastSlash str_browsePath
				if (getDirectories str_browsePath).count > 0 then
				(
					ShellLaunch "explorer.exe" str_browsePath
				)
				else
				(
					messageBox "Error: Path is invalid." title:""
				)
			)
		)
		
		fn f_findMaxfiles path_str =
		(
			local a_files = getFiles (path_str + @"\*.max")
			local a_fileNames = for a in a_files collect getFilenameFile a
			sort a_fileNames
			a_fileNames
		)
			
		fn f_maxfileItemsDisplay path_str =
		(
			if path_str != "" then
			(
				local a_items = f_findMaxfiles path_str
				if a_items.count == 0 do
				(
					a_items = #("*** no maxfiles found ***")
				)
				return a_items
			)
			else
			(
				return #("*** no maxfiles found ***")
			)
		)
		
		fn f_getSceneFile =
		(
			if maxFilePath != "" then
			(
				return (maxFilePath + maxFileName)
			)
			else
			(
				return ("###   no maxfile loaded   ###")
			)
		)
		
		fn f_isNotRootDrive path_str =
		(
			if (filterstring path_str @"\").count == 1 do
			(
				--// It is a root path!
				return false
			)
			true
		)
		
		--// Check if the maxfile version being accessed is newer than the current Max version
		fn f_checkFileVersion file_str =
		(
			local a_fileVersion = getMaxFileVersionData file_str
			--// Note: sometimes the getMaxFileVersionData returns undefined
			if a_fileVersion != undefined do
			(
				if (a_fileVersion)[1] > (maxVersion())[1] do
				(
					return false
				)
			)
			true
		)
		
		fn f_isValidMaxfile mFile_str =
		(
			if (findString mFile_str @".max") != undefined  AND doesFileExist mFile_str do
			(
				return true
			)
			false
		)
		
		fn f_updateHistoryPaths path_str =
		(
			local historyIniFile = (getDir #plugcfg + @"\MaxfilesBrowserHistory.ini")
			local a_historyTags = getIniSetting historyIniFile "PATHS LIST"
			local a_historyPaths = #()
			local a_recentPathItems = #(@"***  RECENT FILES LIST  ***")
			
			--// Collect the history paths from the custom ini file
			for tag in a_historyTags do
			(
				append a_historyPaths ((getIniSetting historyIniFile "PATHS LIST" tag) + @"\")
			)
			
			--// Check if the path already exists in the history list
			local found = false
			local index = 1
			for i = 1 to a_historyPaths.count do
			(
				if (stricmp path_str a_historyPaths[i]) == 0 do
				(
					found = true
					index = i
					exit
				)
			)
			
			--// Add the path to the top of the history list
			append a_recentPathItems path_str
			
			if found then --// Add the item as a unique path to the top of the list
			(
				for i = 1 to a_historyPaths.count where (stricmp a_historyPaths[i] a_historyPaths[index]) != 0 do
				(
					append a_recentPathItems a_historyPaths[i]
				)
			)
			else --// Move the path to the top of the list
			(
				for i = 1 to a_historyPaths.count do
				(
					if i == g_LBTools_MFB.maxHistoryPaths do exit --/ Don't add anything beyond the max item limit
					append a_recentPathItems a_historyPaths[i]
				)
			)
				
			--// Update the ini file with the new list
			delIniSetting historyIniFile "PATHS LIST"
			for i = 2 to a_recentPathItems.count do
			(
				setIniSetting historyIniFile "PATHS LIST" ("PATH:" + ((i-1) as string)) (replace a_recentPathItems[i] a_recentPathItems[i].count 1 "")
			)
			
			--// Set the history paths item index to make sure the correct path is showing after the file is opened (now at the top) - if not browsing the recent files list
			if g_LBTools_MFB.historyPathsIndex != 1 do
			(
				g_LBTools_MFB.historyPathsIndex = 2
			)
		)
		
		fn f_loadFileQuiet fileToLoad =
		(
			if (f_isValidMaxfile fileToLoad) then
			(
				if (f_checkFileVersion fileToLoad) then
				(
					loadMaxFile fileToLoad quiet:true
					--// Update the history method ini file to add the current (opened) scene path if unique, or make it the most recent item
					f_updateHistoryPaths (getFileNamePath fileToLoad)
				)
				else
				(
					messageBox "Error: The selected maxfile is a newer version than the current version - unable to open." title:""
				)
			)
			else
			(
				messageBox "Error: The selected maxfile cannot be found - unable to open." title:""
			)
		)
		
		fn f_mergeFileQuiet fileToMerge =
		(
			if (f_isValidMaxfile fileToMerge) then
			(
				if (f_checkFileVersion fileToMerge) then
				(
					mergeMaxFile fileToMerge #prompt #select quiet:true
				)
				else
				(
					messageBox "Error: The selected maxfile is a newer version than the current version - unable to merge." title:""
				)
			)
			else
			(
				messageBox "Error: The selected maxfile cannot be found - unable to open for merge." title:""
			)
		)
		
		fn f_alternativeLoadMerge quietMode mFile =
		(
			if keyboard.controlPressed do
			(
				if quietMode then
				(
					f_loadFileQuiet mFile
				)
				else
				(
					loadMaxFile mFile quiet:false
				)
			)
			if keyboard.shiftPressed do
			(
				if quietMode then
				(
					f_mergeFileQuiet mFile
				)
				else
				(
					mergeMaxFile mFile #prompt #select quiet:false
				)
			)
		)
			
		fn f_dumpToText a_maxfiles =
		(
			try
			(
				if a_maxfiles.count > 0 do
				(
					makeDir @"C:\Temp"
					local txtFile = @"C:\TEMP\MaxfilesBrowser_Dump.txt"
					local edit_txtFile = openFile txtFile mode:"w"
					for i = 1 to a_maxfiles.count do
					(
						format "%\n" a_maxfiles[i] to:edit_txtFile
					)
					close edit_txtFile
					ShellLaunch txtFile ""
				)
			)catch(messageBox "Error with dump to text file." title:"")
		)
		
		--// Assumes the rootPath_str is not a system root directory (i.e. C:\ or D:\). Must be a subfolder.
		fn f_listRecurseFolderMaxfiles rootPath_str =
		(
			try --// Try catch needed to handle Access Denied error
			(
				local searchSubDirs = (dotnetClass "System.IO.SearchOption")
				local sFiles = dotnetClass "System.IO.Directory"
				local a_maxFiles = sFiles.GetFiles rootPath_str @"*.max"  searchSubDirs.AllDirectories
				sort a_maxFiles
			)catch(a_maxFiles = #())
			a_maxFiles
		)
		
		fn f_subFolderDisplay rootPath a_maxfiles noExt:false =
		(
			local a_subFolderMaxfiles = #()
			for mFile in a_maxfiles do 
			(
				local filePath = getFileNamePath mFile
				if filePath != rootPath then --// Maxfile is in a subfolder
				(
					mFile = @"..\" + (substituteString mFile rootPath  "")
				)					
				else --// Maxfile is in the root path
				(
					mFile = (substituteString mFile rootPath  "")
				)
				if noExt then
				(
					append a_subFolderMaxfiles (getFileNameFile(substituteString mFile rootPath  ""))
				)
				else
				(
					append a_subFolderMaxfiles (substituteString mFile rootPath  "")
				)
			)
			a_subFolderMaxfiles
		)
		
		fn f_filterItems str a_items =
		(
			local a_temp = #()
			for item in a_items do
			(
				if (findString item str) != undefined do
				(
					append a_temp item
				)
			)
			a_temp
		)
		
		fn f_initialDirectory =
		(
			local initialPath = maxFilePath
			if initialPath == "" do
			(
				initialPath = getDir #Scene
			)
			initialPath
		)
		
		--// The script should not interfere with Max accessing the original xml file
		fn f_copyRecentDocsXML =
		(
			local recentDocsXML = (getDir #maxdata + "RecentDocuments.xml")
			local MFBrecentDocsXML = (getDir #plugcfg + "\MFB_RecentDocuments.xml")
			if (doesFileExist MFBrecentDocsXML) do
			(
				deleteFile MFBrecentDocsXML
			)
			copyFile recentDocsXML MFBrecentDocsXML
			MFBrecentDocsXML
		)
		
		--// Allow filtering for subfolders based on the prefix "..\"
		fn f_getSubFilteredItems path_str filter_str a_MFitemsAll =
		(
			local a_MFitemsAllSub = f_subFolderDisplay path_str a_MFitemsAll
			local a_MFitems = (f_filterItems filter_str a_MFitemsAllSub)
			--// Convert back to full paths and files
			for i = 1 to a_MFitems.count do
			(
				if (findString a_MFitems[i] @"..\") != undefined then
				(
					a_MFitems[i] = (replace a_MFitems[i] 1 3 path_str)
				)
				else
				(
					a_MFitems[i] = path_str + a_MFitems[i]
				)
			)
			a_MFitems
		)
		
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/	
		-- //	ABOUT ROLLOUT
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		
		rollout aboutScript_Rollout "About"
		(
			label lbl_author "Author - Lorne Brooks"
			hyperLink myEmail "lbtools2k@gmail.com" color:(color 0 0 255) hovercolor:(color 0 0 190) visitedcolor:(color 190 0 0) align:#center address:"mailto:lbtools2k@gmail.com"
			label lbl_version "Version 1.6 - 13 September 2018"
			
			on aboutScript_Rollout moved pos do
			(
				g_LBTools_MFB.MFB_Pos = pos
				setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "POS" (pos as string)
			)
		) --// end ABOUT rollout

		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/	
		-- //	SETTINGS ROLLOUT
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		
		rollout settings_Rollout "Settings"
		(
			checkbox chkbx_longUI "Longer UI" align:#left tooltip:"Increases the overall length of the UI for displaying more maxfiles at once"
			checkbox chkbx_persistFilters "Persistent Filters" align:#left checked:false tooltip:"A filter string will remain persistent when clicking between browser methods or when reloading the UI"
			checkbox chkbx_quietMode "Quiet Mode" align:#left checked:true tooltip:"If checked: errors like missing maps, MassFX warnings, version mismatch etc will be ignored when opening or merging"
			
			on settings_Rollout open do
			(
				local isLongUI = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "LONG-UI")
				local isPersistFilters = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "PERSIST-FILTERS")
				local isQuietMode = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "QUIET-MODE")
				
				if isLongUI then
					chkbx_longUI.checked = true
				else
					chkbx_longUI.checked = false

				if isPersistFilters then
					chkbx_persistFilters.checked = true
				else
					chkbx_persistFilters.checked = false
				
				if isQuietMode then
					chkbx_quietMode.checked = true
				else
					chkbx_quietMode.checked = false
			)
			
			on chkbx_longUI changed state do
			(
				if state then
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "LONG-UI" "true"
				)
				else
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "LONG-UI" "false"
				)
				g_LBTools_MFB.f_openMaxfilesBrowser()
			)
			
			on chkbx_persistFilters changed state do
			(
				if state then
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "PERSIST-FILTERS" "true"
					g_LBTools_MFB.persistFilters = true
				)
				else
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "PERSIST-FILTERS" "false"
					g_LBTools_MFB.persistFilters = false
				)
			)
			
			on chkbx_quietMode changed state do
			(
				if state then
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "QUIET-MODE" "true"
					g_LBTools_MFB.quietMode = true
				)
				else
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "QUIET-MODE" "false"
					g_LBTools_MFB.quietMode = false
				)
			)
		) --// end SETTINGS rollout
			
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/	
		-- //	TEXTFILE METHOD ROLLOUT
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		rollout textMaxfilesBrowser_Rollout "Specific maxfiles stored in a text file"
		(
			local a_MFitemsAll = #() --// All unfiltered items
			local a_MFitemsLoad = #() --// Items to load into the list based on filtering
			local maxfileIndex = 0
			local textFile = ""
			
			fn f_listMaxfiles =
			(
				if textFile != "" then
				(
					local textFileRead = openFile textFile mode:"r"
					if textFileRead != undefined then
					(
						while NOT eof textFileRead do
						(
							local str_line = readLine textFileRead
							if str_line != "" do --// Ignore blank lines
							(
								append a_MFitemsAll str_line
							)
						)
						close textFileRead
						return a_MFitemsAll
					)
					else --// Nothing in the file
					(
						return #()
					)
				)
				else --// File could not be found
				(
					return #()
				)
			)

			fn f_textDisplay txtFile =
			(
				if txtFile == "" then
				(
					return "HINT: Create a new text file with valid maxfile paths to display in the list box."
				)
				txtFile
			)
			
			fn f_openRollout =
			(
				removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
				addRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			)
			
			label lbl_txt "Text file..." align:#left offset:[5,-3]
			edittext edtxt_path "" readonly:true
			button btn_refresh "Refresh" align:#left across:5 width:60 height:20 tooltip:"Reopens the UI to refresh the list in case the text file has been updated"
			button btn_add "+++   Add Current Scene To List  +++" align:#center offset:[155,0] height:20 offset:[0,0] tooltip:"Adds the current scene maxfile path to the existing specified text file"
			button btn_new "New..." align:#right height:20 offset:[185,0] tooltip:"Create a new text file to store specific maxfile paths"
			button btn_edit "Edit..." align:#right height:20 offset:[85,0] tooltip:"Opens the listed text file for editing"
			button btn_browse "Browse..." align:#right height:20 tooltip:"Find a customized text file that lists the maxfiles to display in the UI for opening or merging"
			button btn_clearFilter "x" width:16 height:16 align:#left offset:[0,3] across:2 tooltip:"Clears the filter string to remove all filtering"
			edittext edtxt_filter "Fillter:" text:("") offset:[-355,4] width:735
			label lbl_txt2 "Maxfiles stored in the chosen text file..." align:#left offset:[5,0]
			listbox lstbx_maxfiles "" height:g_LBTools_MFB.textListHeight selection:0
			button btn_open ">>   OPEN   <<" width:200 height:20 align:#left offset:[120,0] enabled:false across:2 tooltip:"Alternatively: Control+Click on a file to Open"
			button btn_merge ">>   MERGE   <<" width:200 height:20 align:#right offset:[-120,0] enabled:false tooltip:"Alternatively: Shift+Click on a file to Merge"
			group ""
			(
				label lbl_scene "Current scene file:" align:#left offset:[0,5] across:3
				edittext edtxt_scene "" text:("") align:#left width:630 offset:[-160,5] readonly:true
				button btn_folderBrowse "" align:#right offset:[0,0] images:#(g_LBTools_MFB.imageFile1, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an Explorer window at the current scene file location"
			)
			
			on textMaxfilesBrowser_Rollout open do
			(
				if NOT (doesFileExist (getDir #plugcfg + "\MaxfilesBrowserText.ini")) do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserText.ini") "TEXT FILE METHOD" "PATH" ""
				)
				textFile = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserText.ini") "TEXT FILE METHOD" "PATH"
				if NOT (doesFileExist textFile) do
				(
					textFile = ""
				)
				edtxt_path.text = f_textDisplay textFile
				a_MFitemsAll = a_MFitemsLoad = f_listMaxfiles()
				--// Populate the list based on the persistent filters setting
				if g_LBTools_MFB.persistFilters do
				(
					edtxt_filter.text = g_LBTools_MFB.filters_str
					a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
				)
				if a_MFitemsLoad.count == 0 do
				(
					a_MFitemsLoad = #("*** no maxfiles found ***")
				)
				lstbx_maxfiles.items = a_MFitemsLoad
				edtxt_scene.text = f_getSceneFile()
				setFocus edtxt_filter
			)

			on btn_clearFilter pressed do
			(
				g_LBTools_MFB.filters_str = edtxt_filter.text = ""
				f_openRollout()
			)
			
			on edtxt_filter changed str do
			(
				if str != "" then
				(
					g_LBTools_MFB.filters_str = str
					a_MFitemsLoad = (f_filterItems str a_MFitemsAll)
					if a_MFitemsLoad.count == 0 do
					(
						a_MFitemsLoad = #("*** no maxfiles found ***")
					)
					lstbx_maxfiles.items = a_MFitemsLoad
				)
				else --// Filter string is empty
				(
					g_LBTools_MFB.filters_str = ""
					lstbx_maxfiles.items = a_MFitemsLoad = a_MFitemsAll
				)
			)
			
			on lstbx_maxfiles selected index do
			(
				if textFile != "" then
				(
					if doesFileExist textFile then
					(
						if a_MFitemsLoad[1] == "*** no maxfiles found ***" OR a_MFitemsLoad[1] ==undefined then
						(
							f_openRollout()
						)
						else
						(
							if index > 0 do
							(
								maxfileIndex = index
								if (f_isValidMaxfile a_MFitemsLoad[maxfileIndex]) then
								(
									btn_open.enabled = true
									btn_merge.enabled = true
									f_alternativeLoadMerge g_LBTools_MFB.quietMode a_MFitemsLoad[maxfileIndex]
									edtxt_scene.text = f_getSceneFile()
									setFocus edtxt_filter
								)
								else --// Either the maxfile doesn't exist or only a path is entered in the text file (i.e. no .max extension)
								(
									local msgtxt = stringStream ""
									if (findString a_MFitemsLoad[maxfileIndex] @".max") != undefined then
									(
										-- // Keep this format!
										format "Error: Can't find the maxfile...\n\n\t%\n\nPlease review and update the text file (click on Edit).\n\nClick on Refresh when done updating." ((getFilenameFile a_MFitemsLoad[maxfileIndex]) + ".max") to:msgtxt
									)
									else
									(
										-- // Keep this format!
										format "Error: The item selected is not a valid maxfile.\n\nPlease review and update the text file (click on Edit).\n\nClick on Refresh when done updating." to:msgtxt
									)
									messageBox msgtxt title:""
									btn_open.enabled = false
									btn_merge.enabled = false
								)
							)
						)
					)
					else
					(
						messageBox "Error: The loaded text file is missing." title:""
						f_openRollout()
					)
				)
				else
				(
					f_openRollout()
				)
			)
				
			on btn_folderBrowse pressed do
			(
				f_browseScenePath()
				setFocus edtxt_filter
			)
			
			on btn_edit pressed do
			(
				if textFile != "" then
				(
					if doesFileExist textFile then
					(
						ShellLaunch textFile ""
					)
					else
					(
						messageBox "Error: The loaded text file is missing." title:""
						f_openRollout()
					)	
				)
				else
				(
					messageBox "Error: No text file specified to open. Use the Browse button to find one, or the New button to create one." title:""
				)
			)
			
			on btn_refresh pressed do
			(
				f_openRollout()
			)
			
			on btn_add pressed do
			(
				local currentScene = (maxFilePath + maxFileName)
				textFile = edtxt_path.text
				if textFile != "" then
				(
					if doesFileExist textFile then
					(
						if currentScene != "" then
						(
							local edit_txtFile = openFile textFile mode:"r+"
							if edit_txtFile != undefined do
							(
								local isUnique = true
								while NOT eof edit_txtFile do
								(
									local str_line = readLine edit_txtFile
									if str_line != "" do --// Ignore blank lines
									(
										if str_line == currentScene do
										(
											isUnique = false
										)
									)
								)
								if isUnique then
								(
									format "%\n" currentScene to:edit_txtFile
								)
								else
								(
									messageBox "Current scene file is already in the list." title:""
								)
								close edit_txtFile
								f_openRollout()
							)
						)
						else
						(
							messageBox "Error: Scene file not yet loaded." title:""
						)
					)
					else
					(
						messageBox "Error: The loaded text file is missing." title:""
						f_openRollout()
					)
				)
				else
				(
					messageBox "Error: Text file not yet loaded - click the [New...] button to create one." title:""
				)
			)
			
			on btn_new pressed do
			(
				makeDir @"C:\Temp"
				textFile = @"C:\TEMP\TextFileMethod.txt"
				local makeNewFile = true
				if (doesFileExist textFile) do
				(
					if NOT (queryBox "Text file already exists - do you want to overwrite it?" title:"") do
					(
						makeNewFile = false
					)
				)
				if makeNewFile do
				(
					local edit_txtFile = openFile textFile mode:"w"
					close edit_txtFile
					if textFile != undefined do
					(
						setIniSetting (getDir #plugcfg + "\MaxfilesBrowserText.ini") "TEXT FILE METHOD" "PATH" textFile
					)
					f_openRollout()
					ShellLaunch textFile ""
				)
				setFocus edtxt_filter
			)
			
			on btn_browse pressed do
			(
				textFile = getOpenFileName caption:"Open a text file list of maxfiles..." filename:@"C:\TEMP\" types:"(*.txt)"
				if textFile != undefined do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserText.ini") "TEXT FILE METHOD" "PATH" textFile
				)
				f_openRollout()
			)
			
			on btn_open pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						f_loadFileQuiet a_MFitemsLoad[maxfileIndex]
					)
					else
					(
						loadMaxFile (a_MFitemsLoad[maxfileIndex]) quiet:false
					)				
				)
				edtxt_scene.text = f_getSceneFile()
				setFocus edtxt_filter
			)
			
			on btn_merge pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						f_mergeFileQuiet a_MFitemsLoad[maxfileIndex]
					)
					else
					(
						mergeMaxFile (a_MFitemsLoad[maxfileIndex]) #prompt #select quiet:true
					)				
				)
				setFocus edtxt_filter
			)
		) --// end TEXT rollout

		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		-- //	FOLDER METHOD ROLLOUT
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/		
		rollout folderMaxfilesBrowser_Rollout "Specific folders containing maxfiles"
		(
			local a_MFitemsAll = #() --// All unfiltered items
			local a_MFitemsLoad = #() --// Items to load into the list based on filtering
			local maxfileIndex = 0
			
			fn f_pathItemsDisplay =
			(
				local pathsCount = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS"
				local a_paths = #()
				if execute(pathsCount) > 0 then
				(
					for pc = 1 to execute(pathsCount) do
					(
						append a_paths (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" ("PATH:" + (pc as string)))
					)
				)
				else
				(
					append a_paths "*** no paths stored ***"
				)
				return a_paths
			)
			
			fn f_findCurrentPathIndex =
			(
				local pathsCount = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS"
				if execute(pathsCount) > 0 then
				(
					local currentPath = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH"
					local a_paths = #()
					-- // Collect all the stored paths
					for pc = 1 to execute(pathsCount) do
					(
						append a_paths (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" ("PATH:" + (pc as string)))
					)
					-- // Compare them all to the current path to find the matching one
					for p = 1 to a_paths.count do
					(
						if (matchPattern a_paths[p] pattern:currentPath) do
						(
							-- // This becomes the listbox selection index value
							return p
						)
					)
				)
				else
				(
					return 0
				)
			)
			
			fn f_doesFolderPathExist =
			(
				local currentPath = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH"
				local a_storedPaths = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST"
				for i in a_storedPaths do
				(
					if (matchPattern (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" i) pattern:currentPath) do
					(
						return true
					)
				)
				return false
			)
			
			fn f_removeEntry indexToRemove =
			(
				-- // Remove the chosen entry
				delIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" ("PATH:" + (indexToRemove as string))
				-- // Update the paths counter
				local pathsCount = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS") - 1
				setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS" (pathsCount as string)
				-- // Rebuild the paths list to make the path numbers are sequential again, by storing the remaining paths
				local a_remainingPaths = #()
				a_remainingPaths = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" 
				local a_pathContents = #()
				for i in a_remainingPaths do
				(
					append a_pathContents (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" i)
				)
				-- // Clear all remaining paths from the list in the INI file
				delIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST"
				-- // Now add them back to the INI file
				for i = 1 to a_pathContents.count do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" ("PATH:" + (i as string)) a_pathContents[i]
				)
				-- // Default back to the last entry in the selection box
				if a_pathContents.count > 0 do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH" a_pathContents[a_pathContents.count]
				)
				if a_remainingPaths.count == 0 do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH" ""
				)
			)
			
			fn f_verifyPathsList currPath =
			(
				-- // Gather the paths from the INI into an array
				local a_existingPaths = #()
				a_existingPaths = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" 
				local a_pathContents = #()
				for i in a_existingPaths do
				(
					append a_pathContents (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" i)
				)
				-- // Can't use GetDirectories while there's a backslash on the last character in a path, when checking whether a folder exists. Need to remove the backslash first
				local a_pathsNoSlash = #()
				for c in a_pathContents do
				(
					append a_pathsNoSlash (f_removeLastSlash c)
				)
				-- // See if the directories actually exist, returns false if one doesn't
				for p in a_pathsNoSlash do
				(
					if (getDirectories p).count == 0 do
					(
						return p
					)
				)
				return ""
			)
			
			fn f_findMissingPathIndex missingPath_str =
			(
				local a_pathsList = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST"
				local a_pathContents = #()
				for p in a_pathsList do
				(
					append a_pathContents (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" p)
				)
				for i = 1 to a_pathContents.count do
				(
					if (matchPattern a_pathContents[i] pattern:missingPath_str) do
					(
						return i -- // found the missing path index
					)
				)
			)
				
			fn f_openRollout =
			(
				removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
				addRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			)
			
			button btn_addPath "Add" offset:[0,0] across:4 align:#left width:60 height:20 tooltip:"Add a new folder containing maxfiles to the list"
			button btn_removePath "Remove" align:#left offset:[-125,0] width:60 height:20 enabled:false tooltip:"Remove the selected folder from the list"
			button btn_dumpToText "***  Dump To Text File  ***" offset:[-95,0] height:20 align:#center tooltip:"Maxfiles found in the selected folder path will be output to a textfile as a list here... C:\\TEMP\\MaxfilesBrowser_Dump.txt"
			button btn_explorePath "Explore Path" align:#right offset:[0,0] width:90 height:20 enabled:true tooltip:"Opens an Explorer window at the selected folder path"
			label lbl_txt1 "Stored paths..." align:#left offset:[5,0]
			listbox lstbx_paths "" height:g_LBTools_MFB.foldersPathsHeight items:#("") selection:0
			button btn_clearFilter "x" width:16 height:16 align:#left offset:[0,3] across:2 tooltip:"Clears the filter string to remove all filtering"
			edittext edtxt_filter "Fillter:" text:("") offset:[-355,5] width:735
			label lbl_txt2 "Maxfilles in the chosen stored path or its subfolders..." align:#left across:2 offset:[5,4]
			checkbox chkbx_subfolders "Recurse subfolders" align:#right offset:[0,4] checked:true tooltip:"If checked, the maxfiles in the subfolders for the current path will also be included"
			listbox lstbx_maxfiles "" height:g_LBTools_MFB.foldersListHeight items:#("") selection:0
			button btn_open ">>   OPEN   <<" width:200 height:20 align:#left offset:[120,0] enabled:false across:2 tooltip:"Alternatively: Control+Click on a file to Open"
			button btn_merge ">>   MERGE   <<" width:200 height:20 align:#right offset:[-120,0] enabled:false tooltip:"Alternatively: Shift+Click on a file to Merge"
			group ""
			(
				label lbl_scene "Current scene file:" align:#left offset:[0,5] across:3
				edittext edtxt_scene "" text:("") align:#left width:630 offset:[-160,5] readonly:true
				button btn_folderBrowse "" align:#right offset:[0,0] images:#(g_LBTools_MFB.imageFile1, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an Explorer window at the current scene file location"
			)
			
			on folderMaxfilesBrowser_Rollout open do
			(
				if NOT (doesFileExist (getDir #plugcfg + "\MaxfilesBrowserFolders.ini")) do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "RECURSE STATE" "SUBFOLDERS" "TRUE"
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS" "0"
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH" ""
				)
				-- // Initialize the interface
				lstbx_paths.items = folderMaxfilesBrowser_Rollout.f_pathItemsDisplay()
				lstbx_paths.selection = folderMaxfilesBrowser_Rollout.f_findCurrentPathIndex()
				local recurseState = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "RECURSE STATE" "SUBFOLDERS"
				local currentPath = (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH")
				if execute(recurseState) then
				(	
					if currentPath != "" then
					(
						-- // Make sure the paths actually exist
						local missingPath = f_verifyPathsList currentPath
						if missingPath == "" then
						(
							-- // The dotNet recurse function will not accept a system root directory (e.g. C:\ or D:\). Files must be in subfolders.
							if (f_isNotRootDrive currentPath) then
							(
								a_MFitemsAll = a_MFitemsLoad = f_listRecurseFolderMaxfiles currentPath
								--// Populate the list based on the persistent filters setting
								if g_LBTools_MFB.persistFilters do
								(
									edtxt_filter.text = g_LBTools_MFB.filters_str
									a_MFitemsLoad = f_getSubFilteredItems currentPath g_LBTools_MFB.filters_str a_MFitemsAll
								)
								if a_MFitemsLoad.count == 0 then
								(
									a_MFitemsLoad = #("*** no maxfiles found ***")
									lstbx_maxfiles.items = a_MFitemsLoad
								)
								else
								(
									lstbx_maxfiles.items = f_subFolderDisplay currentPath a_MFitemsLoad
								)
								chkbx_subfolders.checked = true
								setFocus edtxt_filter
							)
							else
							(
								messageBox "Error: The current scene exists in a system root directory... (e.g. C:\\ or D:\\ )\n\nPlease move it to a subfolder." title:""
							)	
						)
						else -- // If not, remove the offending entry from the paths list
						(
							local msgtxt = stringStream ""
							-- // Keep this format!
							format "Error: Can't find the actual path ...\n\n>>>   %  <<<\n\nThis has been removed from the paths list." missingPath to:msgtxt
							messageBox msgtxt title:""
							missingPath = missingPath + "\\"
							f_removeEntry (f_findMissingPathIndex missingPath)
							f_openRollout()
						)
					)
					else
					(
						lstbx_maxfiles.items = a_MFitemsLoad = #("*** no maxfiles found ***")
					)
				)
				else --// Root path only
				(
					a_MFitemsAll = (f_maxfileItemsDisplay currentPath)
					--// Populate the list based on the persistent filters setting
					if g_LBTools_MFB.persistFilters do
					(
						edtxt_filter.text = g_LBTools_MFB.filters_str
						a_MFitemsAll = (f_maxfileItemsDisplay currentPath)
						a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
					)
					if a_MFitemsLoad.count == 0 do
					(
						a_MFitemsLoad = #("*** no maxfiles found ***")
					)
					lstbx_maxfiles.items = a_MFitemsLoad
					chkbx_subfolders.checked = false
					setFocus edtxt_filter
				)
				if lstbx_paths.selection > 0 then
				(
					btn_removePath.enabled = true
				)
				else
				(
					btn_removePath.enabled = false
				)
				edtxt_scene.text = f_getSceneFile()
				setFocus edtxt_filter
			)
			
			on btn_clearFilter pressed do
			(
				g_LBTools_MFB.filters_str = edtxt_filter.text = ""
				f_openRollout()
			)
			
			on edtxt_filter changed str do
			(
				local path_str = (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH")
				if str != "" then
				(
					g_LBTools_MFB.filters_str = str
					if chkbx_subfolders.checked then
					(
						a_MFitemsLoad = f_getSubFilteredItems path_str str a_MFitemsAll
						if a_MFitemsLoad.count == 0 then
						(
							a_MFitemsLoad = #("*** no maxfiles found ***")
							lstbx_maxfiles.items = a_MFitemsLoad
						)
						else
						(
							lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
						)
					)
					else
					(
						a_MFitemsLoad = (f_filterItems str a_MFitemsAll)
						if a_MFitemsLoad.count == 0 do
						(
							a_MFitemsLoad = #("*** no maxfiles found ***")
						)
						lstbx_maxfiles.items = a_MFitemsLoad
					)
				)
				else --// Filter string is empty
				(
					g_LBTools_MFB.filters_str = ""
					if chkbx_subfolders.checked then
					(
 						a_MFitemsLoad = a_MFitemsAll
						lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsAll
					)
					else
					(					
						lstbx_maxfiles.items = a_MFitemsLoad = a_MFitemsAll						
					)
				)
			)
			
			on btn_explorePath pressed do
			(
				local str_browsePath = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH"
				str_browsePath = f_removeLastSlash str_browsePath
				if str_browsePath != "" AND (getDirectories str_browsePath).count > 0 then
				(
					ShellLaunch "explorer.exe" str_browsePath
				)
				else
				(
					messageBox "Error: Path is invalid." title:""
				)
				setFocus edtxt_filter
			)
			
			on btn_folderBrowse pressed do
			(
				f_browseScenePath()
				setFocus edtxt_filter
			)
			
			on chkbx_subfolders changed state do
			(
				if state then
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "RECURSE STATE" "SUBFOLDERS" "TRUE"
				)
				else
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "RECURSE STATE" "SUBFOLDERS" "FALSE"
				)
				f_openRollout()
			)
			
			on lstbx_paths selected index do
			(
				if index > 0 do
				(
					local pickedPath = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" ("PATH:" + (index as string))
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH" pickedPath
					lstbx_paths.selection = index
					f_openRollout()
				)
			)
		
			on btn_dumpToText pressed do
			(
				local path_str = lstbx_paths.selected
				local a_outputFormat = a_MFitemsAll
				--// Convert the format of non-recursed maxfiles to full paths
				if chkbx_subfolders.checked == false do
				(
					for i = 1 to a_outputFormat.count do
					(
						a_outputFormat[i] = path_str + a_outputFormat[i] + ".max"
					)
				)
				f_dumpToText a_outputFormat
				setFocus edtxt_filter
			)
			
			on btn_addPath pressed do
			(
				-- // Picks a new folder path
				--// Note: getSavePath will crash is ESC is pressed - use the Cancel button instead
				local userPath_str = getSavePath caption:"Choose a folder containing maxfiles..." initialDir:(f_initialDirectory())
				if userPath_str != "" AND userPath_str != undefined do -- // If false, it means the user cancelled the dialog
				(
					-- // Do not allow C:\ or D:\ system root directories to be added, as they conflict with the dotNet recurse diretories function
					if (userPath_str != @"C:\") AND (userPath_str != @"D:\") then
					(
						userPath_str = userPath_str + "\\"
						-- // Update the current path
						setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH" userPath_str
						if (f_doesFolderPathExist()) == false then
						(
							-- // Read the total number of paths stored
							local pathsCount = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS"
							-- // Add the new path to the paths list
							setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS LIST" ("PATH:" + ((execute(pathsCount) + 1) as string)) userPath_str
							-- // Update the paths counter
							setIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS" ((execute(pathsCount) + 1) as string)
							btn_removePath.enabled = true
							lstbx_paths.selection = execute(pathsCount) + 1
							f_openRollout()
						)
						else
						(
							messageBox "Error: The chosen path already exists in the paths list." title:""
						)
					)
					else
					(
						messageBox "Error: The path you picked was a system root directory... ( C:\\ or D:\\ )\n\nThis is not allowed. Please choose a subfolder instead." title:""
					)
				)
			)
			
			on btn_removePath pressed do
			(
				if (queryBox "Are you sure you want to remove the folder path?" title:"") do
				(
					f_removeEntry lstbx_paths.selection
					local pathsCount = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "PATHS COUNT" "NUMBER OF PATHS")
					if pathsCount == 0 do
					(
						btn_removePath.enabled = false
					)
					lstbx_paths.selection = pathsCount
				)
				f_openRollout()
			)
			
			on lstbx_maxfiles selected index do
			(
				if index > 0 do
				(
					if (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH") != "" then
					(
						if a_MFitemsLoad[1] == "*** no maxfiles found ***" OR a_MFitemsLoad[1] == undefined then
						(
							f_openRollout()
						)
						else
						(
							maxfileIndex = index
							btn_open.enabled = true
							btn_merge.enabled = true
							local mFile
							if chkbx_subfolders.checked then
							(							
								mFile = a_MFitemsLoad[maxfileIndex]
							)
							else
							(
								mFile = (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH") + "\\" + a_MFitemsLoad[maxfileIndex] + ".max"
							)
							f_alternativeLoadMerge g_LBTools_MFB.quietMode mFile
							edtxt_scene.text = f_getSceneFile()
						)
					)
					else
					(
						f_openRollout()
					)
					setFocus edtxt_filter
				)
			)
				
			on btn_open pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						if chkbx_subfolders.checked then
						(
							f_loadFileQuiet a_MFitemsLoad[maxfileIndex]
						)
						else
						(
							f_loadFileQuiet ((getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH") + "\\" + a_MFitemsLoad[maxfileIndex] + ".max")
						)
					)
					else
					(
						if chkbx_subfolders.checked then
						(
							loadMaxFile a_MFitemsLoad[maxfileIndex] quiet:false
						)
						else
						(
							loadMaxFile ((getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH") + "\\" + a_MFitemsLoad[maxfileIndex] + ".max") quiet:false
						)
					)			
				)
				edtxt_scene.text = f_getSceneFile()
				setFocus edtxt_filter
			)
			
			on btn_merge pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						if chkbx_subfolders.checked then
						(
							f_mergeFileQuiet a_MFitemsLoad[maxfileIndex]
						)
						else
						(
							f_mergeFileQuiet ((getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH") + "\\" + a_MFitemsLoad[maxfileIndex] + ".max")
						)
					)
					else
					(
						if chkbx_subfolders.checked then
						(
							mergeMaxFile a_MFitemsLoad[maxfileIndex] #prompt #select quiet:false
						)
						else
						(
							mergeMaxFile ((getIniSetting (getDir #plugcfg + "\MaxfilesBrowserFolders.ini") "CURRENT PATH" "PATH") + "\\" + a_MFitemsLoad[maxfileIndex] + ".max") #prompt #select quiet:false
						)
					)				
				)
				setFocus edtxt_filter
			)
		) --// end FOLDER rollout

		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		-- //	SCENE METHOD ROLLOUT
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		rollout sceneMaxfilesBrowser_Rollout "Maxfiles in the same folder as the current scene file"
		(
			local a_MFitemsAll = #() --// All unfiltered items
			local a_MFitemsLoad = #() --// Items to load into the list based on filtering
			local maxfileIndex = 0
			local overrideBrowse = false
				
			fn f_sceneTextDisplay path_str =
			(
				if path_str == "" then
				(
					return "HINT: Open a maxfile scene to display in the listbox other maxfiles found in the same folder."
				)
				path_str
			)
			
			fn f_itemsDisplay path_str =
			(
				if path_str == "" then
				(
					return #("*** no maxfiles found ***")
				)
				f_findMaxfiles path_str
			)

			fn f_getMaxFileToLoad path_str =
			(
				local recurseState = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserScene.ini") "RECURSE STATE" "SUBFOLDERS"
				if  execute(recurseState) then
				(
					return a_MFitemsLoad[maxfileIndex]
				)
				(path_str + a_MFitemsLoad[maxfileIndex] + ".max")
			)
			
			fn f_verifyCurrentPathExists path_str =
			(
				if (getFiles (path_str + "*.max")).count > 0 do
				(
					return true
				)
				false
			)
			
			fn f_saveOrResetScene =
			(
				if (queryBox "Error: The current file or path cannot be found, but you have working data still open.\n\nDo you want to SAVE your current scene to a new file?\n\nChoosing NO will RESET the scene." title:"") then
				(
					local newSaveFile = getSaveFileName filename:".max" caption:"Save Maxfile As..." types:"3ds Max (*.max)"
					if newSaveFile != undefined then
					(
						saveMaxFile newSaveFile
						loadMaxFile newSaveFile
					)
					else
					(
						resetMaxFile()
					)
				)
				else
				(
					resetMaxFile()
				)
			)
			
			fn f_displayParentPathList parentPath =
			(
				-- // The dotNet recurse function will not accept a system root directory (e.g. C:\ or D:\). Files must be in subfolders.
				if (f_isNotRootDrive parentPath) then
				(
					sceneMaxfilesBrowser_Rollout.edtxt_path.text = parentPath
						
					--// Update the list based on the new path and recurse state
					local recurseState = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserScene.ini") "RECURSE STATE" "SUBFOLDERS"
					if execute(recurseState) then
					(
						sceneMaxfilesBrowser_Rollout.edtxt_path.text = parentPath
						if (f_isNotRootDrive parentPath) then
						(
							a_MFitemsAll = a_MFitemsLoad = f_listRecurseFolderMaxfiles parentPath
							if a_MFitemsAll.count > 0 then
							(
								if g_LBTools_MFB.persistFilters do
								(
									a_MFitemsLoad = f_getSubFilteredItems parentPath g_LBTools_MFB.filters_str a_MFitemsAll
								)
								if a_MFitemsLoad.count == 0 then
								(
									a_MFitemsLoad = #("*** no maxfiles found ***")
									sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsLoad
								)
								else
								(
									sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = f_subFolderDisplay parentPath a_MFitemsLoad
								)
								maxfileIndex = sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.selection
								sceneMaxfilesBrowser_Rollout.btn_open.enabled = false
								sceneMaxfilesBrowser_Rollout.btn_merge.enabled = false
								sceneMaxfilesBrowser_Rollout.chkbx_subfolders.checked = true
							)
							else
							(
								sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsAll = a_MFitemsLoad = #("*** no maxfiles found ***")
							)
						)
						else
						(
							messageBox "Error: The current scene exists in a system root directory... (e.g. C:\\ or D:\\ )\n\nPlease move it to a subfolder." title:""
						)
					)
					else --// Root path only
					(
						a_MFitemsAll = a_MFitemsLoad = f_itemsDisplay parentPath
						if a_MFitemsAll.count > 0 then
						(
							if g_LBTools_MFB.persistFilters do
							(
								a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
							)
							if a_MFitemsLoad.count == 0 do
							(
								a_MFitemsLoad = #("*** no maxfiles found ***")
							)
							sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsLoad
							maxfileIndex = sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.selection
							sceneMaxfilesBrowser_Rollout.btn_open.enabled = false
							sceneMaxfilesBrowser_Rollout.btn_merge.enabled = false
							sceneMaxfilesBrowser_Rollout.chkbx_subfolders.checked = false
						)
						else
						(
							sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsAll = a_MFitemsLoad = #("*** no maxfiles found ***")
						)
					)
				)
				else
				(
					messageBox "Error: The path string cannot be a system root path.\n\ne.g. C:\\ or D:\\" title:""
				)
			)
			
			fn f_loadItemsNoFilter path_str =
			(
				a_MFitemsLoad = a_MFitemsAll
				if sceneMaxfilesBrowser_Rollout.chkbx_subfolders.checked then
				(
					if a_MFitemsLoad.count == 0 then
					(
						a_MFitemsLoad = #("*** no maxfiles found ***")
						sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsLoad
					)
					else
					(
						sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
					)
				)
				else
				(
					if a_MFitemsLoad.count == 0 do
					(
						a_MFitemsLoad = #("*** no maxfiles found ***")
					)
					sceneMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsLoad
				)
			)
			
			fn f_openRollout =
			(
				removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
				addRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			)
			
			button btn_refresh "Refresh" align:#left across:3 width:60 height:20 tooltip:"Reopens the UI to refresh the maxfiles list in case the listed current scene path is different from the open scene path"
			button btn_dumpToText "***  Dump To Text File  ***" offset:[1,0] height:20 align:#center tooltip:"Maxfiles found in the current scene path will be output to a textfile as a list here... C:\\TEMP\\MaxfilesBrowser_Dump.txt"
			button btn_browse "Browse..." align:#right height:20 tooltip:"Browse for a path containing maxfiles to display in the list for opening or merging - temporary override of open scene path"
			label lbl_txt1 "Current scene path..." align:#left offset:[5,0]
			button btn_parentPath "UP" align:#left offset:[7,1] width:22 height:22 across:2 images:#(g_LBTools_MFB.imageFile2, undefined, 1, 1, 1, 1, 1, false) tooltip:"Browse for maxfiles in the parent folder (i.e. move up one subfolder) - temporary override of open scene path"
			edittext edtxt_path "" text:("") align:#left offset:[-350,3] width:731 readonly:true
			button btn_clearFilter "x" width:16 height:16 align:#left offset:[0,3] across:2 tooltip:"Clears the filter string to remove all filtering"
			edittext edtxt_filter "Fillter:" text:("") offset:[-355,5] width:735
			label lbl_txt2 "Maxfilles found in the current scene path or its subfolders..." align:#left offset:[5,0] across:2
			checkbox chkbx_subfolders "Recurse subfolders" align:#right offset:[0,1] checked:true tooltip:"If checked, the maxfiles in the subfolders for the current scene folder will also be included"
			listbox lstbx_maxfiles "" height:g_LBTools_MFB.sceneListHeight items:#() selection:maxfileIndex
			button btn_open ">>   OPEN   <<" width:200 height:20 align:#left offset:[120,0] enabled:false across:2 tooltip:"Alternatively: Control+Click on a file to Open"
			button btn_merge ">>   MERGE   <<" width:200 height:20 align:#right offset:[-120,0] enabled:false tooltip:"Alternatively: Shift+Click on a file to Merge"
			group ""
			(
				label lbl_scene "Current scene file:" align:#left offset:[0,5] across:3
				edittext edtxt_scene "" text:("") align:#left width:630 offset:[-160,5] readonly:true
				button btn_folderBrowse "" align:#right offset:[0,0] images:#(g_LBTools_MFB.imageFile1, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an Explorer window at the current scene file location"
			)
			
			on sceneMaxfilesBrowser_Rollout open do
			(
				if NOT (doesFileExist (getDir #plugcfg + "\MaxfilesBrowserScene.ini")) do
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserScene.ini") "RECURSE STATE" "SUBFOLDERS" "TRUE"
				)
				local currentScenePath = edtxt_path.text = f_sceneTextDisplay maxFilePath
				if maxFilePath != "" then
				(
					if (f_verifyCurrentPathExists currentScenePath) then
					(
						local recurseState = getIniSetting (getDir #plugcfg + "\MaxfilesBrowserScene.ini") "RECURSE STATE" "SUBFOLDERS"
						if execute(recurseState) then
						(
							-- // The dotNet recurse function will not accept a system root directory (e.g. C:\ or D:\). Files must be in subfolders.
							if (f_isNotRootDrive maxFilePath) then
							(
								a_MFitemsAll = a_MFitemsLoad = f_listRecurseFolderMaxfiles maxFilePath
								if g_LBTools_MFB.persistFilters do
								(
									edtxt_filter.text = g_LBTools_MFB.filters_str
									a_MFitemsLoad = f_getSubFilteredItems maxFilePath g_LBTools_MFB.filters_str a_MFitemsAll
								)
								if a_MFitemsLoad.count == 0 then
								(
									a_MFitemsLoad = #("*** no maxfiles found ***")
									lstbx_maxfiles.items = a_MFitemsLoad
								)
								else
								(
									lstbx_maxfiles.items = f_subFolderDisplay currentScenePath a_MFitemsLoad
								)
								maxfileIndex = lstbx_maxfiles.selection
								btn_open.enabled = false
								btn_merge.enabled = false
								chkbx_subfolders.checked = true
							)
							else
							(
								messageBox "Error: The current scene exists in a system root directory... (e.g. C:\\ or D:\\ )\n\nPlease move it to a subfolder." title:""
							)
						)
						else --// Root path only
						(
							a_MFitemsAll = a_MFitemsLoad = f_itemsDisplay maxFilePath
							--// Populate the list based on the persistent filters setting
							if g_LBTools_MFB.persistFilters do
							(
								edtxt_filter.text = g_LBTools_MFB.filters_str
								a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
							)
							if a_MFitemsLoad.count == 0 do
							(
								a_MFitemsLoad = #("*** no maxfiles found ***")
							)
							lstbx_maxfiles.items = a_MFitemsLoad
							maxfileIndex = lstbx_maxfiles.selection
							btn_open.enabled = false
							btn_merge.enabled = false
							chkbx_subfolders.checked = false
						)
					)
					else -- // Because the scene has data in it, we should prompt the user to save their data to a new file
					(
						f_saveOrResetScene()
						f_openRollout()
					)
				)
				else
				(
					edtxt_filter.text = g_LBTools_MFB.filters_str
					lstbx_maxfiles.items = a_MFitemsLoad = #("*** no maxfiles found ***")
				)
				edtxt_scene.text = f_getSceneFile()
				setFocus edtxt_filter
			)

			on btn_clearFilter pressed do
			(
				--// We don't want clearing the filter to behave like Refresh, as you may have used the Up arrow or Browse button
				--// to temporarily override the current scene path (i.e. don't reload the UI)
				g_LBTools_MFB.filters_str = edtxt_filter.text = ""
				local path_str = edtxt_path.text
				f_loadItemsNoFilter path_str
			)
			
			on edtxt_filter changed str do
			(
				local path_str = edtxt_path.text
				if path_str != "" do
				(
					if str != "" then
					(
						g_LBTools_MFB.filters_str = str
						if chkbx_subfolders.checked then
						(
							a_MFitemsLoad = f_getSubFilteredItems path_str str a_MFitemsAll
							if a_MFitemsLoad.count == 0 then
							(
								a_MFitemsLoad = #("*** no maxfiles found ***")
								lstbx_maxfiles.items = a_MFitemsLoad
							)
							else
							(
								lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
							)
						)
						else
						(
							a_MFitemsLoad = (f_filterItems str a_MFitemsAll)
							if a_MFitemsLoad.count == 0 do
							(
								a_MFitemsLoad = #("*** no maxfiles found ***")
							)
							lstbx_maxfiles.items = a_MFitemsLoad
						)
					)
					else --// Filter string is empty
					(
						g_LBTools_MFB.filters_str = ""
						f_loadItemsNoFilter path_str
					)
				)
			)
			
			--// Note: This is a temporary override of the Scene method default behavior of only displaying the current scene path maxfiles and subfolder files (Up Arrow was clicked)
			on btn_parentPath pressed do
			(
				local currentScenePath = edtxt_path.text
				local directoryCheck = replace currentScenePath currentScenePath.count 1 "" --// Remove the last backslash
				
				--// Check that the scene path directory string is valid
				if (getDirectories directoryCheck).count > 0 do
				(
					--// Move up one subfolder if possible
					local a_subFolders = filterString currentScenePath @"\"
					if a_subFolders.count > 1 do
					(
						local parentPath = ""
						for i = 1 to (a_subFolders.count - 1) do
						(
							parentPath += a_subFolders[i] + @"\"
						)
						f_displayParentPathList parentPath
					)
				)
				edtxt_filter.text = g_LBTools_MFB.filters_str
				if NOT g_LBTools_MFB.persistFilters do
				(
					edtxt_filter.text = g_LBTools_MFB.filters_str = ""
				)
				setFocus edtxt_filter
			)

			--// Note: This is a temporary override of the Scene method default behavior of only displaying the current scene path maxfiles and subfolder files
			on btn_browse pressed do
			(
				--// Note: getSavePath will crash is ESC is pressed - use the Cancel button instead
				local newPath = getSavePath caption:"Open a text file list of maxfiles..." initialDir:(f_initialDirectory())
				if newPath != "" AND newPath != undefined do
				(
					newPath = newPath + @"\"
					f_displayParentPathList newPath
					overrideBrowse = true
				)
				edtxt_filter.text = g_LBTools_MFB.filters_str
				if NOT g_LBTools_MFB.persistFilters do
				(
					edtxt_filter.text = g_LBTools_MFB.filters_str = ""
				)
				setFocus edtxt_filter
			)
			
			on btn_refresh pressed do
			(
				f_openRollout()
			)
			
			on btn_folderBrowse pressed do
			(
				f_browseScenePath()
				setFocus edtxt_filter
			)
			
			on lstbx_maxfiles selected index do
			(				
				if index > 0 do
				(
					if a_MFitemsLoad[1] == "*** no maxfiles found ***" OR a_MFitemsLoad[1] == undefined then
					(
						if edtxt_path.text == maxFilePath then
						(
							f_openRollout()
						)
						else --// The parent path button was clicked
						(
							lstbx_maxfiles.selection = 0
						)
					)
					else
					(
						--// The list of maxfiles may not be in sync with the current scene file path, as the Browse button may have been used (hence the overrideBrowse flag)
						if maxFilePath != "" OR overrideBrowse then
						(
							local currentScenePath = edtxt_path.text
							maxfileIndex = index
							local mFile
							
							if chkbx_subfolders.checked then
							(
								--// The dotNet recurse function will not accept a system root directory (e.g. C:\ or D:\). Files must be in subfolders.
								if (f_isNotRootDrive currentScenePath) then
								(
									if a_MFitemsLoad[maxfileIndex] != undefined do
									(
										mFile = a_MFitemsLoad[maxfileIndex]
									)
								)
								else
								(
									messageBox "Error: The current scene exists in a system root directory... (e.g. C:\\ or D:\\ )\n\nPlease move it to a subfolder." title:""
								)	
							)
							else
							(
								if a_MFitemsLoad[maxfileIndex] != undefined do
								(
									mFile = currentScenePath + a_MFitemsLoad[maxfileIndex] + ".max"
								)
							)
							
							if mFile != undefined then
							(
								if doesFileExist mFile then
								(
									btn_open.enabled = true
									btn_merge.enabled = true
									if keyboard.controlPressed OR keyboard.shiftPressed do
									(
										btn_open.enabled = false
										btn_merge.enabled = false
										lstbx_maxfiles.selection = 0
										f_alternativeLoadMerge g_LBTools_MFB.quietMode mFile
										f_openRollout()
									)
								)
								else -- // Because the scene has data in it, we should prompt the user to save their data to a new file
								(
									f_saveOrResetScene()
									f_openRollout()
								)
							)
							else
							(
								f_openRollout()
							)
						)
						else
						(
							f_openRollout()
						)
					)
					setFocus edtxt_filter
				)
			)
			
			on chkbx_subfolders changed state do
			(
				if state then
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserScene.ini") "RECURSE STATE" "SUBFOLDERS" "TRUE"
				)
				else
				(
					setIniSetting (getDir #plugcfg + "\MaxfilesBrowserScene.ini") "RECURSE STATE" "SUBFOLDERS" "FALSE"
				)
				local parentPath = edtxt_path.text
				
				--// First check if the path is valid
				if (getDirectories (replace parentPath parentPath.count 1 "")).count > 0 do
				(
					if parentPath == maxFilePath then
					(
						f_openRollout()
					)
					else --// Either the up arrow was pressed or the browse button was used and is a valid new directory
					(
						if state then --// Recurse subfolders
						(
							a_MFitemsAll = a_MFitemsLoad = f_listRecurseFolderMaxfiles parentPath
							if a_MFitemsAll.count > 0 then
							(
								if g_LBTools_MFB.persistFilters do
								(
									a_MFitemsLoad = f_getSubFilteredItems parentPath g_LBTools_MFB.filters_str a_MFitemsAll
								)
								lstbx_maxfiles.items = f_subFolderDisplay parentPath a_MFitemsLoad
								maxfileIndex = lstbx_maxfiles.selection
								btn_open.enabled = false
								btn_merge.enabled = false
								chkbx_subfolders.checked = true
							)
							else
							(
								lstbx_maxfiles.items = a_MFitemsAll = a_MFitemsLoad = #("*** no maxfiles found ***")
							)
						)
						else --// Root path only
						(
							--// Update the list for the new path
							local a_MFitemsAll = f_itemsDisplay parentPath
							if a_MFitemsAll.count > 0 then
							(
								if g_LBTools_MFB.persistFilters do
								(
									a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
								)
								lstbx_maxfiles.items = a_MFitemsLoad
								maxfileIndex = lstbx_maxfiles.selection
								btn_open.enabled = false
								btn_merge.enabled = false
								chkbx_subfolders.checked = false
							)
							else
							(
								lstbx_maxfiles.items = a_MFitemsAll = a_MFitemsLoad = #("*** no maxfiles found ***")
							)
						)
					)
				)
			)
			
			on btn_dumpToText pressed do
			(
				local path_str = maxFilePath
				local a_outputFormat = a_MFitemsAll
				--// Convert the format of non-recursed maxfiles to full paths
				if chkbx_subfolders.checked == false do
				(
					for i = 1 to a_outputFormat.count do
					(
						a_outputFormat[i] = path_str + a_outputFormat[i] + ".max"
					)
				)
				f_dumpToText a_outputFormat
				setFocus edtxt_filter
			)
			
			on btn_open pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						f_loadFileQuiet (f_getMaxFileToLoad edtxt_path.text)
					)
					else
					(
						loadMaxFile (f_getMaxFileToLoad edtxt_path.text) quiet:false
					)
					f_openRollout()
				)
			)
			
			on btn_merge pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						f_mergeFileQuiet (f_getMaxFileToLoad edtxt_path.text)
					)
					else
					(
						mergeMaxFile (f_getMaxFileToLoad edtxt_path.text) #prompt #select quiet:false
					)
					f_openRollout()
				)
				setFocus edtxt_filter
			)
		) --// end SCENE rollout

		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		-- //	HISTORY METHOD ROLLOUT
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		rollout historyMaxfilesBrowser_Rollout "Most recent maxfiles"
		(
			local a_MFitemsAll = #() --// All unfiltered items
			local a_MFitemsLoad = #() --// Items to load into the list based on filtering
			local a_recentPathItems = #()
			local maxfileIndex = 0
			
			fn f_getHistoryFiles =
			(
				local historyFile = f_copyRecentDocsXML()
				local a_files = #()
				
				if historyFile != "" then
				(
					local fileToRead = openFile historyFile mode:"r"
					while NOT eof fileToRead do
					(
						local str_line = readLine fileToRead
						if str_line != "" do --// Ignore blank lines
						(
							if (findString str_line "<FilePath>") != undefined do
							(
								a_filteredLine = filterString str_line "<>"
								append a_files a_filteredLine[3]
							)
						)
					)
				)
				else
				(
					return #()
				)
				a_files
			)
			
			fn f_getHistoryPathsDefault =
			(
				local maxIni = getDir #maxData + @"3dsmax.ini"
				local a_historyTags = getIniSetting maxIni "MaxHistoryList"
				local a_items = #()
				
				for i = 2 to a_historyTags.count do
				(
					local path_str = (getIniSetting maxIni "MaxHistoryList" a_historyTags[i])
					append a_items path_str
				)
				a_items
			)
			
			fn f_getHistoryPaths =
			(
				local historyIniFile = (getDir #plugcfg + @"\MaxfilesBrowserHistory.ini")
				a_recentPathItems = #(@"***  RECENT FILES LIST  ***")
				
				--// Pre-load the list with whatever items are already in 3dsmax.ini history list if the custom ini file doesn't exist yet (first time usage)
				if NOT doesFileExist historyIniFile then
				(
					local a_defaultPaths = f_getHistoryPathsDefault()
					
					for i = 1 to a_defaultPaths.count do
					(
						setIniSetting historyIniFile "PATHS LIST" ("PATH:" + ((i-1) as string)) a_defaultPaths[i]
					)
				)
				
				--// Populate the list from the custom history paths ini file
				local a_historyTags = getIniSetting historyIniFile "PATHS LIST"
				for i = 1 to a_historyTags.count do
				(
					local path_str = (getIniSetting historyIniFile "PATHS LIST" a_historyTags[i])
					append a_recentPathItems (path_str + @"\")
				)
			)
			
			fn f_loadSubfolderItems path_str =
			(
				a_MFitemsAll = a_MFitemsLoad = f_listRecurseFolderMaxfiles path_str
				if a_MFitemsAll.count > 0 then
				(
					if g_LBTools_MFB.persistFilters do
					(
						a_MFitemsLoad = f_getSubFilteredItems path_str g_LBTools_MFB.filters_str a_MFitemsAll
					)
					if a_MFitemsLoad.count == 0 then
					(
						a_MFitemsLoad = #("*** no maxfiles found ***")
						historyMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsLoad
					)
					else
					(
						historyMaxfilesBrowser_Rollout.lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
					)
				)
				else
				(
					historyMaxfilesBrowser_Rollout.lstbx_maxfiles.items = #("*** no maxfiles found ***")
				)
			)
			
			fn f_loadRootItems path_str =
			(
				a_MFitemsAll = a_MFitemsLoad = f_maxfileItemsDisplay path_str
				if g_LBTools_MFB.persistFilters do
				(
					a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
				)
				if a_MFitemsLoad.count == 0 do
				(
					a_MFitemsLoad = #("*** no maxfiles found ***")
				)
				historyMaxfilesBrowser_Rollout.lstbx_maxfiles.items = a_MFitemsLoad
			)
			
			fn f_openRollout =
			(
				removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
				addRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			)
			
			button btn_dumpToText "***  Dump To Text File  ***" offset:[0,0] height:20 align:#center tooltip:"Maxfiles found in the recent files will be output to a textfile as a list here... C:\\TEMP\\MaxfilesBrowser_Dump.txt"
			dropdownlist ddl_paths "" items:a_recentPathItems height:100
			button btn_clearFilter "x" width:16 height:16 align:#left offset:[0,-1] across:2 tooltip:"Clears the filter string to remove all filtering"
			edittext edtxt_filter "Fillter:" text:("") offset:[-355,-1] width:735
			label lbl_txt2 "Maxfiles stored in the most recent list..." align:#left offset:[0,2] across:2
			checkbox chkbx_subfolders "Recurse subfolders" align:#right offset:[0,0] enabled:true checked:true tooltip:"If checked, any maxfiles in the subfolders for the selected folder path will also be included"
			listbox lstbx_maxfiles "" height:g_LBTools_MFB.historyListHeight items:#() selection:maxfileIndex
			button btn_open ">>   OPEN   <<" width:200 height:20 align:#left offset:[120,0] enabled:false across:2 tooltip:"Alternatively: Control+Click on a file to Open"
			button btn_merge ">>   MERGE   <<" width:200 height:20 align:#right offset:[-120,0] enabled:false tooltip:"Alternatively: Shift+Click on a file to Merge"
			group ""
			(
				label lbl_scene "Current scene file:" align:#left offset:[0,5] across:3
				edittext edtxt_scene "" text:("") align:#left width:630 offset:[-160,5] readonly:true
				button btn_folderBrowse "" align:#right offset:[0,0] images:#(g_LBTools_MFB.imageFile1, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an Explorer window at the current scene file location"
			)
			
			on historyMaxfilesBrowser_Rollout open do
			(
				if g_LBTools_MFB.historySubfolders then
				(
					chkbx_subfolders.checked = true
				)
				else
				(
					chkbx_subfolders.checked = false
				)
				if g_LBTools_MFB.historyPathsIndex > 1 then
				(
					chkbx_subfolders.enabled = true
				)
				else
				(
					chkbx_subfolders.enabled = false
				)
				
				f_getHistoryPaths()				
				ddl_paths.items = a_recentPathItems
				
				if g_LBTools_MFB.historyPathsIndex == 1 then --// Show most recent history files
				(
					a_MFitemsAll = a_MFitemsLoad = f_getHistoryFiles()
					if g_LBTools_MFB.persistFilters do
					(
						edtxt_filter.text = g_LBTools_MFB.filters_str
						a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
					)
					if a_MFitemsLoad.count == 0 do
					(
						a_MFitemsLoad = #("*** no maxfiles found ***")
					)
					lstbx_maxfiles.items = a_MFitemsLoad
				)
				else --// Show items in the chosen history path
				(
					local path_str = a_recentPathItems[g_LBTools_MFB.historyPathsIndex]
					if g_LBTools_MFB.historySubfolders then --// Show all files in all subfolders
					(
						a_MFitemsAll = a_MFitemsLoad = f_listRecurseFolderMaxfiles path_str
						if g_LBTools_MFB.persistFilters do
						(
							edtxt_filter.text = g_LBTools_MFB.filters_str
							a_MFitemsLoad = f_getSubFilteredItems path_str g_LBTools_MFB.filters_str a_MFitemsAll
						)
						if a_MFitemsLoad.count == 0 then
						(
							a_MFitemsLoad = #("*** no maxfiles found ***")
							lstbx_maxfiles.items = a_MFitemsLoad
						)
						else
						(
							lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
						)
					)
					else --// Root path only
					(
						a_MFitemsLoad = a_MFitemsAll = f_maxfileItemsDisplay path_str
						if g_LBTools_MFB.persistFilters do
						(
							edtxt_filter.text = g_LBTools_MFB.filters_str
							a_MFitemsLoad = (f_filterItems g_LBTools_MFB.filters_str a_MFitemsAll)
						)
						if a_MFitemsLoad.count == 0 do
						(
							a_MFitemsLoad = #("*** no maxfiles found ***")
						)
						lstbx_maxfiles.items = a_MFitemsLoad
					)
					ddl_paths.selection = g_LBTools_MFB.historyPathsIndex
				)
				edtxt_scene.text = f_getSceneFile()
				setFocus edtxt_filter
			)
			
			on btn_clearFilter pressed do
			(
				g_LBTools_MFB.filters_str = edtxt_filter.text = ""
				f_openRollout()
			)
			
			on btn_folderBrowse pressed do
			(
				f_browseScenePath()
				setFocus edtxt_filter
			)
			
			on edtxt_filter changed str do
			(
				g_LBTools_MFB.historyPathsIndex = ddl_paths.selection
				local path_str = a_recentPathItems[g_LBTools_MFB.historyPathsIndex]
					
				if str != "" then
				(
					g_LBTools_MFB.filters_str = str
					if g_LBTools_MFB.historyPathsIndex == 1 then
					(
						a_MFitemsLoad = (f_filterItems str a_MFitemsAll)
						if a_MFitemsLoad.count == 0 do
						(
							a_MFitemsLoad = #("*** no maxfiles found ***")
						)
						lstbx_maxfiles.items = a_MFitemsLoad
					)
					else
					(
						if chkbx_subfolders.checked then
						(
							a_MFitemsLoad = f_getSubFilteredItems path_str str a_MFitemsAll
							if a_MFitemsLoad.count == 0 then
							(
								a_MFitemsLoad = #("*** no maxfiles found ***")
								lstbx_maxfiles.items = a_MFitemsLoad
							)
							else
							(
								lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
							)
						)
						else
						(
							a_MFitemsLoad = (f_filterItems str a_MFitemsAll)
							if a_MFitemsLoad.count == 0 do
							(
								a_MFitemsLoad = #("*** no maxfiles found ***")
							)
							lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad noExt:true
						)
					)
				)
				else --// Filter string is empty
				(
					g_LBTools_MFB.filters_str = ""
					if g_LBTools_MFB.historyPathsIndex == 1 then --// Recent files list
					(
						lstbx_maxfiles.items = a_MFitemsLoad = a_MFitemsAll
					)
					else --// A selected recent folder
					(
						a_MFitemsLoad = a_MFitemsAll
						if chkbx_subfolders.checked then
						(
							lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad
						)
						else
						(
							lstbx_maxfiles.items = f_subFolderDisplay path_str a_MFitemsLoad noExt:true
						)
					)
				)
			)
			
			on btn_dumpToText pressed do
			(
				f_dumpToText a_MFitemsAll
				setFocus edtxt_filter
			)
			
			on ddl_paths selected val do
			(
				g_LBTools_MFB.historyPathsIndex = val
				if val == 1 then --// Load the default recent files list
				(
					f_openRollout()
				)
				else --// Populate the recent files list with whatever files are in the chosen path (ignoring subfolders)
				(
					local path_str = a_recentPathItems[val]
					if g_LBTools_MFB.historySubfolders then --// Show all files in all subfolders
					(
						f_loadSubfolderItems path_str
					)
					else --// Root path only
					(
						f_loadRootItems path_str
					)
					chkbx_subfolders.enabled = true
					setFocus edtxt_filter
				)
			)
			
			on chkbx_subfolders changed state do
			(
				local path_str = a_recentPathItems[g_LBTools_MFB.historyPathsIndex]
				if state then --// Show all files in all subfolders
				(
					g_LBTools_MFB.historySubfolders = true
					f_loadSubfolderItems path_str
				)
				else --// Root path only
				(
					g_LBTools_MFB.historySubfolders = false
					f_loadRootItems path_str
				)
				setFocus edtxt_filter
			)
			
			on lstbx_maxfiles selected index do
			(
				if index > 0 do
				(
					if a_MFitemsLoad[1] == "*** no maxfiles found ***" OR a_MFitemsLoad[1] == undefined then
					(
						f_openRollout()
					)
					else
					(
						maxfileIndex = index
						if (doesFileExist (a_MFitemsLoad[maxfileIndex])) then
						(
							btn_open.enabled = true
							btn_merge.enabled = true
							f_alternativeLoadMerge g_LBTools_MFB.quietMode a_MFitemsLoad[maxfileIndex]
							if keyboard.controlPressed do
							(
								f_openRollout()
							)
						)
						else
						(
							messageBox "Error: Can't find the maxfile.\n" title:""
							btn_open.enabled = false
							btn_merge.enabled = false				
						)
					)
					setFocus edtxt_filter
				)
			)
			
			on btn_open pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						f_loadFileQuiet a_MFitemsLoad[maxfileIndex]
					)
					else
					(
						loadMaxFile (a_MFitemsLoad[maxfileIndex]) quiet:false
					)
					f_openRollout()
				)
			)
			
			on btn_merge pressed do
			(
				if maxfileIndex != 0 do
				(
					if g_LBTools_MFB.quietMode then
					(
						f_mergeFileQuiet a_MFitemsLoad[maxfileIndex]
					)
					else
					(
						mergeMaxFile (a_MFitemsLoad[maxfileIndex]) #prompt #select quiet:true
					)				
				)
				setFocus edtxt_filter
			)
		) --// end HISTORY rollout
		
		
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		-- //	MAIN ROLLOUT DEFINITION
		/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		checkbutton chkbtn_historyMethod "HISTORY" across:4 width:180 height:20 tooltip:"Access the most recently opened maxfiles"
		checkbutton chkbtn_sceneMethod "SCENE" width:180 height:20 tooltip:"Access maxfiles located in the same folder as the current scene file"
		checkbutton chkbtn_folderMethod "FOLDERS" width:180 height:20 tooltip:"Access collections of maxfiles based on stored folder paths"
		checkbutton chkbtn_textMethod "FILES" width:180 height:20 tooltip:"Access individually stored maxfiles saved as favorites" --// Internally referred to as "text"
			
		on maxfilesBrowserUI_Rollout open do
		(
			methodStored = getIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD"
			case methodStored of
			(
				"history":(
					addRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					chkbtn_historyMethod.checked = true
					chkbtn_sceneMethod.checked = false
					chkbtn_folderMethod.checked = false
					chkbtn_textMethod.checked = false
				)
				"scene":(
					removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					addRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					chkbtn_historyMethod.checked = false	
					chkbtn_sceneMethod.checked = true
					chkbtn_folderMethod.checked = false
					chkbtn_textMethod.checked = false
				)
				"folder":(
					removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater	
					addRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater					
					removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					chkbtn_historyMethod.checked = false
					chkbtn_sceneMethod.checked = false
					chkbtn_folderMethod.checked = true
					chkbtn_textMethod.checked = false
				)
				"text":(
					removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater	
					removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					addRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
					chkbtn_historyMethod.checked = false
					chkbtn_sceneMethod.checked = false
					chkbtn_folderMethod.checked = false
					chkbtn_textMethod.checked = true
				)
			)
		)
		
		on chkbtn_historyMethod changed state do
		(
			chkbtn_historyMethod.checked = true
			chkbtn_sceneMethod.checked = false
			chkbtn_folderMethod.checked = false
			chkbtn_textMethod.checked = false
			addRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD" "history"
			setFocus historyMaxfilesBrowser_Rollout.edtxt_filter
		)
			
		on chkbtn_sceneMethod changed state do
		(
			chkbtn_historyMethod.checked = false
			chkbtn_sceneMethod.checked = true
			chkbtn_folderMethod.checked = false
			chkbtn_textMethod.checked = false
			removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			addRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD" "scene"
			setFocus sceneMaxfilesBrowser_Rollout.edtxt_filter
		)
			
		on chkbtn_folderMethod changed state do
		(
			chkbtn_historyMethod.checked = false
			chkbtn_sceneMethod.checked = false
			chkbtn_folderMethod.checked = true
			chkbtn_textMethod.checked = false
			removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater	
			addRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater					
			removeRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD" "folder"
			setFocus folderMaxfilesBrowser_Rollout.edtxt_filter
		)
		
		on chkbtn_textMethod changed state do
		(
			chkbtn_historyMethod.checked = false
			chkbtn_sceneMethod.checked = false
			chkbtn_folderMethod.checked = false
			chkbtn_textMethod.checked = true
			removeRollout historyMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			removeRollout sceneMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater	
			removeRollout folderMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			addRollout textMaxfilesBrowser_Rollout g_LBTools_MFB.maxfilesBrowserFloater
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD" "text"
			setFocus textMaxfilesBrowser_Rollout.edtxt_filter
		)
	) --// end rollout

	fn f_openMaxfilesBrowser =
	(
		-- // For first time usage
		if NOT (doesFileExist (getDir #plugcfg + "\MaxfilesBrowser.ini")) do
		(
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "POS" ((mouse.screenpos + [-75,-15]) as string)
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD" "history"
		)
		if NOT (doesFileExist (getDir #plugcfg + "\MaxfilesBrowserSettings.ini")) do
		(
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "LONG-UI" "false"
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "PERSIST-FILTERS" "false"
			setIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "QUIET-MODE" "true"
		)
		
		--// Retrieve and assign the Settings information
		local useLongUI = execute (getIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "LONG-UI")
		if useLongUI then
		(
			g_LBTools_MFB.UIheight = 1073
			g_LBTools_MFB.textListHeight = 57
			g_LBTools_MFB.foldersPathsHeight = 10
			g_LBTools_MFB.foldersListHeight = 47
			g_LBTools_MFB.sceneListHeight = 56
			g_LBTools_MFB.historyListHeight = 58
		)
		else
		(
			g_LBTools_MFB.UIheight = 670
			g_LBTools_MFB.textListHeight = 26
			g_LBTools_MFB.foldersPathsHeight = 5
			g_LBTools_MFB.foldersListHeight = 21
			g_LBTools_MFB.sceneListHeight = 25
			g_LBTools_MFB.historyListHeight = 27
		)
		g_LBTools_MFB.persistFilters = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "PERSIST-FILTERS")
		g_LBTools_MFB.quietMode = execute(getIniSetting (getDir #plugcfg + "\MaxfilesBrowserSettings.ini") "SETTINGS" "QUIET-MODE")
		
		-- // Setup floater definition
		try(closeRolloutFloater g_LBTools_MFB.maxfilesBrowserFloater)catch()
		g_LBTools_MFB.maxfilesBrowserFloater = newRolloutFloater "Maxfiles Browser" 800 g_LBTools_MFB.UIheight
		g_LBTools_MFB.MFB_Pos = execute (getIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "POS")
		g_LBTools_MFB.maxfilesBrowserFloater.pos = g_LBTools_MFB.MFB_Pos
		addRollout g_LBTools_MFB.maxfilesBrowserUI_Rollout.aboutScript_Rollout g_LBTools_MFB.maxfilesBrowserFloater rolledup:true
		addRollout g_LBTools_MFB.maxfilesBrowserUI_Rollout.settings_Rollout g_LBTools_MFB.maxfilesBrowserFloater rolledup:true
		addRollout g_LBTools_MFB.maxfilesBrowserUI_Rollout g_LBTools_MFB.maxfilesBrowserFloater
		
		local methodStored = getIniSetting (getDir #plugcfg + "\MaxfilesBrowser.ini") "MAXFILES BROWSER FLOATER" "METHOD"
		case methodStored of
		(
			"history":(
				setFocus g_LBTools_MFB.maxfilesBrowserUI_Rollout.historyMaxfilesBrowser_Rollout.edtxt_filter
			)
			"scene":(
				setFocus g_LBTools_MFB.maxfilesBrowserUI_Rollout.sceneMaxfilesBrowser_Rollout.edtxt_filter
			)
			"folder":(
				setFocus g_LBTools_MFB.maxfilesBrowserUI_Rollout.folderMaxfilesBrowser_Rollout.edtxt_filter
			)
			"text":(
				setFocus g_LBTools_MFB.maxfilesBrowserUI_Rollout.textMaxfilesBrowser_Rollout.edtxt_filter
			)
		)
	)
	
	-----------------------------------------------------------------------------------------
	--// Initial calling function
	-----------------------------------------------------------------------------------------
	f_openMaxfilesBrowser()
) --// end function

) --// end struct

g_LBTools_MFB = s_LBTools_maxfilesBrowser()


/*********************************************************************************************************************************************************************/
--// MACRO CALLS FOR UI
/*********************************************************************************************************************************************************************/

macroscript maxfilesBrowser
	category:"LB Tools"
	buttontext:"{ MFB }"
	icon:#("LBTools_MFB",1)
	tooltip:"Maxfiles Browser 1.6"
(
	g_LBTools_MFB.f_maxfilesBrowser()
)

/*********************************************************************************************************************************************************************/